/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrcodehandler.BcdSepaHandler;

import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.frankmuenster.jameica.qrscanner.utils.IbanAndBicValidator;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Klasse f&uuml;r ein BCD Transfer Object. Das Object enth&auml;lt alle Felder,
 * die im BCD-SCT Standard definiert sind und ein Methode zum Erstellen einer
 * Auslandsueberweisung in Hibiscus
 * 
 * @author munstfr
 * @version 1.4
 * 
 */
public class BcdDataTransferObject {

	/**
	 * Die Servicekennung des SPEACredirTransfer Objects. Muss BCT lauten.
	 */
	private String serviceKennung = "BCD";
	/**
	 * Die Version des SPEACredirTransfer Objects.
	 */
	private String version = "";
	/**
	 * Die verwendete Kodierung des SPEACredirTransfer Objects. Dabei gilt: <br/>
	 * 1: UTF-8<br/>
	 * 2: ISO 8895-1<br/>
	 * 3: ISO 8895-2<br/>
	 * 4: ISO 8895-4<br/>
	 * 5: ISO 8895-5<br/>
	 * 6: ISO 8895-7<br/>
	 * 7: ISO 8895-10 und<br/>
	 * 8: ISO 8895-15
	 */
	private String kodierung = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String funktion = "SCT";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String bic = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String empfaenger = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String iban = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String betrag = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String geschaeftsCode = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String referenze = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String verwendungszweck = "";
	/**
	 * Die des SPEACredirTransfer Objects
	 */
	private String benutzerhinweis = "";

	/**
	 * Erzeugt ein neues BcdDataTransferObject
	 */
	public BcdDataTransferObject() {
		super();
	}

	/**
	 * @return the serviceKennung
	 */
	public String getServiceKennung() {
		return serviceKennung;
	}

	/**
	 * @param serviceKennung
	 *            the serviceKennung to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die Servicekennung nicht der Definition
	 *             entspricht. Es ist nur der Wert 'BCD' erlaubt.
	 */
	public void setServiceKennung(String serviceKennung)
			throws BcdDataTransferObjcetDefinitionException {
		if (serviceKennung == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Servicekennung darf nicht null sein.");
		if (!serviceKennung.toUpperCase().contentEquals("BCD"))
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Servicekennung muss 'BCD' lauten");
		this.serviceKennung = serviceKennung.toUpperCase();
	}

	/**
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * @param version
	 *            the version to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die Version null ist.
	 */
	public void setVersion(String version)
			throws BcdDataTransferObjcetDefinitionException {
		if (version == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Version darf nicht null sein.");
		if (!version.equals("001") && !version.equals("002"))
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Version kann nur 001 oder 002 sein.");
		this.version = version;
	}

	/**
	 * @return the kodierung
	 */
	public String getKodierung() {
		return kodierung;
	}

	/**
	 * @param kodierung
	 *            the kodierung to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die Kodierung nicht der Definition
	 *             entspricht (Ziffer zwischen 0 und 8)
	 */
	public void setKodierung(String kodierung)
			throws BcdDataTransferObjcetDefinitionException {

		if (kodierung == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Kodierung darf nicht null sein.");
		try {
			Integer kod = Integer.decode(kodierung);
			if (kod >= 0 || kod <= 8) {
				this.kodierung = String.valueOf(kod);
				return;
			}
		} catch (Exception e) {
			throw new BcdDataTransferObjcetDefinitionException("Die Kodierung "
					+ kodierung
					+ " ist fehlerhaft. Muss eine Ziffer zwischen 0 udn 8 sein");
		}
		throw new BcdDataTransferObjcetDefinitionException("Die Kodierung "
				+ kodierung
				+ " ist fehlerhaft. Muss eine Ziffer zwischen 0 udn 8 sein");
	}

	/**
	 * @return the funktion
	 */
	public String getFunktion() {
		return funktion;
	}

	/**
	 * @param funktion
	 *            the funktion to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die Funktion nicht der Definition
	 *             entspricht. Zur Zeit ist nur 'SCT' als Funktionstring erlaubt
	 */
	public void setFunktion(String funktion)
			throws BcdDataTransferObjcetDefinitionException {
		if (funktion == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Funktion darf nicht null sein.");

		if (funktion.toUpperCase().contentEquals("SCT")) {
			this.funktion = funktion.toUpperCase();
			return;
		}
		throw new BcdDataTransferObjcetDefinitionException(
				"Die Kodierung muss 'SCT' sein");
	}

	/**
	 * @return the bic
	 */
	public String getBic() {
		return bic;
	}

	/**
	 * @param bic
	 *            the BIC to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die BIC nicht der Definition entspricht
	 */
	public void setBic(String bic)
			throws BcdDataTransferObjcetDefinitionException {
		if (bic == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die BIC darf nich NULL sein.");

		if (bic.equals("") && this.version.equals("002")) {
			this.bic = bic;
		} else {

			if (IbanAndBicValidator.isBIC(bic.toUpperCase()) == IbanAndBicValidator.NOT_A_BIC)
				throw new BcdDataTransferObjcetDefinitionException(
						"Die BIC hat ein falsches Format.");
			this.bic = bic.toUpperCase();
		}
	}

	/**
	 * @return the empfaenger
	 */
	public String getEmpfaenger() {
		return empfaenger;
	}

	/**
	 * @param empfaenger
	 *            the empfaenger to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn der Empfnger mehr als 70 zeichen lang
	 *             ist.
	 */
	public void setEmpfaenger(String empfaenger)
			throws BcdDataTransferObjcetDefinitionException {
		if (empfaenger == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Der Empfnger darf nicht null sein.");

		if (empfaenger.length() > 70)
			throw new BcdDataTransferObjcetDefinitionException(
					"Der Empfnger darf nicht mehr als 70 Zeichen lang sein.");
		this.empfaenger = empfaenger;
	}

	/**
	 * @return the iban
	 */
	public String getIban() {
		return iban;
	}

	/**
	 * @param iban
	 *            the iban to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die IBAN nicht der Definition entspricht
	 */
	public void setIban(String iban)
			throws BcdDataTransferObjcetDefinitionException {
		if (iban == null)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die IBAN darf nicht null sein.");

		int result = IbanAndBicValidator.isIBAN(iban.toUpperCase());
		if (result == IbanAndBicValidator.NOT_AN_IBAN)
			throw new BcdDataTransferObjcetDefinitionException(
					"IBAN hat kein gltiges Format.");
		if (result == IbanAndBicValidator.BAD_IBAN_CHECKNUM)
			throw new BcdDataTransferObjcetDefinitionException(
					"Checksumme ser IBAn ist ungltig.");

		this.iban = iban.toUpperCase();
	}

	/**
	 * @return the betrag
	 */
	public String getBetrag() {
		return betrag;
	}

	/**
	 * @param betragString
	 *            the betrag to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn der Betrag nicht der Definition entspricht
	 */
	public void setBetrag(String betragString)
			throws BcdDataTransferObjcetDefinitionException {
		if (betragString == null)
			betragString = "";
		if (!betragString.isEmpty()) {
			if (betragString.toUpperCase().startsWith("EUR")) {
				betragString = betragString.substring(3);
			}
			try {
				Pattern p = Pattern.compile("\\d{1,9}(.\\d{1,2})?");
				Matcher m = p.matcher(betragString);
				boolean isNumber = m.find();
				if (!isNumber)
					throw new BcdDataTransferObjcetDefinitionException(
							"Betrag definition stimmt nicht mit Format 999999999.99 berein");

			} catch (Exception e) {
				throw new BcdDataTransferObjcetDefinitionException(
						"Betrag-Definition stimmt nicht");
			}
		}
		this.betrag = betragString;
	}

	/**
	 * @return the geschaeftsCode
	 */
	public String getGeschaeftsCode() {
		return geschaeftsCode;
	}

	/**
	 * @param geschaeftsCode
	 *            the geschaeftsCode to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn der Geschftscode nicht der Definition
	 *             entspricht
	 */
	public void setGeschaeftsCode(String geschaeftsCode)
			throws BcdDataTransferObjcetDefinitionException {

		if (geschaeftsCode == null)
			geschaeftsCode = "";
		if (!geschaeftsCode.isEmpty() && geschaeftsCode.length() != 4)
			throw new BcdDataTransferObjcetDefinitionException(
					"Geschaeftscode muss 4 Zeichen lang sein.");
		this.geschaeftsCode = geschaeftsCode;
	}

	/**
	 * @return the referenze
	 */
	public String getReferenze() {
		return referenze;
	}

	/**
	 * @param referenze
	 *            the referenze to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn die Refernz zu lang (mehr als 35 Zeichen)
	 *             oder der Verwendungszweck bereits gesetzt ist
	 */
	public void setReferenze(String referenze)
			throws BcdDataTransferObjcetDefinitionException {
		if (referenze == null)
			referenze = "";

		if (referenze.length() > 35)
			throw new BcdDataTransferObjcetDefinitionException(
					"Die Referenze darf nicht mehr als 35 Zeichen lang sein. ");
		//MANTISBT-0000001: Erlaubtz nun mit Warnung
//		if (!referenze.isEmpty() && this.verwendungszweck.length() > 0)
//			throw new BcdDataTransferObjcetDefinitionException(
//					"Dir Referenze kann nur gesetzt werden, wenn der Verwendungszweck leer ist.");
		this.referenze = referenze;

	}

	/**
	 * @return the verwendungszweck
	 */
	public String getVerwednungszweck() {
		return verwendungszweck;
	}

	/**
	 * @param verwendungszweck
	 *            the verwendungszweck to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn der Verwendungszweck zu lang (mehr als 140
	 *             Zeichen) oder die Referenz bereits gesetzt ist
	 */
	public void setVerwendungszweck(String verwendungszweck)
			throws BcdDataTransferObjcetDefinitionException {
		if (verwendungszweck == null)
			verwendungszweck = "";
		if (verwendungszweck.length() > 140)
			throw new BcdDataTransferObjcetDefinitionException(
					"Der Verwendungszweck darf nicht mehr als 140 Zeichen lang sein.");
		// MANTISBT-0000001. Erlaubt nun mit Warnung
//		if (!verwendungszweck.isEmpty() && this.referenze.length() > 0)
//			throw new BcdDataTransferObjcetDefinitionException(
//					"Der Verwendungszweck kann nur gesetzt werden, wenn die Referenze leer ist.");
		this.verwendungszweck = verwendungszweck;
	}

	/**
	 * @return the benutzerhinweis
	 */
	public String getBenutzerhinweis() {
		return benutzerhinweis;
	}

	/**
	 * @param benutzerhinweis
	 *            the benutzerhinweis to set
	 * @throws BcdDataTransferObjcetDefinitionException
	 *             wird erzeugt, wenn der Benutzerhnweis mehr als 70 Zeichen
	 *             lang ist
	 */
	public void setBenutzerhinweis(String benutzerhinweis)
			throws BcdDataTransferObjcetDefinitionException {
		if (benutzerhinweis == null)
			benutzerhinweis = "";
		if (benutzerhinweis.length() > 70)
			throw new BcdDataTransferObjcetDefinitionException(
					"Der Benutzhinweis darf nicht mehr als 70 Zeichen lang sein.");
		this.benutzerhinweis = benutzerhinweis;
	}

	/**
	 * Erstellt eine neue
	 * {@link de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung} und ffent diese
	 * in einer
	 * {@link de.willuhn.jameica.hbci.gui.views.AuslandsUeberweisungNew}
	 * 
	 */
	public void oeffneNeueSEPA() {

		try {
			AuslandsUeberweisung au = null;

			au = (AuslandsUeberweisung) Settings.getDBService().createObject(
					AuslandsUeberweisung.class, null);

			au.setGegenkontoName(getEmpfaenger());
			au.setGegenkontoNummer(getIban());
			au.setGegenkontoBLZ(getBic());
			try {
				Method method = au.getClass().getMethod("setPurposeCode",
						String.class);
				method.invoke(au, getGeschaeftsCode());
			} catch (Exception e) {
				// dann eben nicht
			}
			if (!getBetrag().isEmpty())
				au.setBetrag(Double.valueOf(getBetrag().replace(",", ".")));
			
			// MANTISBT-0000001: Behandlung der Kunden-Referenz bzw. End-to-End ID
			if (!getReferenze().isEmpty() && !getVerwednungszweck().isEmpty()) {
				// zeige Benutzerhinweis
				String text = "Der QR-Code ist fehlerhaft!!!\n\n" +
						"Er enthlt sowohl eine Refernzangebe als auch einen Verwendungszweck.\n" + 
						"Dies ist laut Spezifikation nicht zulssig. Bitte informieren sie den\n" +
						"Rechungsersteller.\n\n" +
						"Die Informationen knnen zusammengefasst als Verwendungszweck bernommen\n" + 
						"werden. Da dies zu einer ungltigen berweisung fhren kann, berprfen\n" +
						"sie  die bernommen Daten bitte genau.\n\n\n" +
						"Daten als Verwendungszweck bernehmen?\n\n ";
				boolean takeboth = Application.getCallback().askUser(text, false);
				if (takeboth) {
					au.setZweck(getReferenze() + " - " + getVerwednungszweck());
				}
				
			} else if (!getReferenze().isEmpty()) {
				au.setZweck(getReferenze());
				au.setEndtoEndId(getReferenze());
				au.setPmtInfId(getReferenze());
			} else {
				au.setZweck(getVerwednungszweck());
			}
			if (au != null) {
				// View neu zeichnen
				GUI.startView(
						de.willuhn.jameica.hbci.gui.views.AuslandsUeberweisungNew.class,
						au);
				if (getBenutzerhinweis().length() > 0) {
					// zeige Benutzerhinweis
					String text = "Hinweise des QrCode-Erstellers:\n\n"
							+ getBenutzerhinweis();
					Application.getCallback().notifyUser(text);
				}
			}
		} catch (RemoteException e) {
			Logger.error("Die SEPA-berweisung konnte nicht erstellt werden. Fehler:\n"
					+ e.getMessage());
		} catch (Exception e) {
			Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n"
					+ e.getMessage());
		}
	}
}

/***********************************************************
 * History: $Log BcdcdDataTransferObject.java,v$ $Date:: 2014-04-01 00:56:03$
 * Date of last commit
 * 
 * @Revision 1.4 Date: 05/07/2016 - Referenz, wenn vorhanden, in PmtInfId eintragen
 * 				Erlaube, mit Warnung, Referenz und Zweckangabe
 * 
 * @Revision 1.2 Date: 20/12/2014 - Wenn die Referenz angegeben, wird versucht
 *           die End-to-End zu setzten. - Geschftscode wird nun auch
 *           verarbeitet.
 * 
 * @Revision 1.1 Date: 21/07/2014 Lngenberprfung des Verwendungszweck falsch.
 *           War 35, korrigiert auf 140.
 * 
 *************************************************************/

