/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrcodehandler.BcdSepaHandler;

import java.util.ArrayList;
import java.util.Arrays;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface
 * {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem
 * Handler fr BCD SEPA berweisungen.
 * 
 * @author munstfr
 * @version 1.4
 * 
 */
public class BcdSepa_001_SCT implements IQrCodeHandler {

	static {
		QrCodeHandlerRegsistry.register(new BcdSepa_001_SCT());
	}

	@Override
	public void handleQrCode(String qrCode) throws QrCodeHandlerException {

		Logger.debug("BCD Code Version 001 - SCT");

		String qrCodeLow = qrCode.toLowerCase();
		if (!qrCodeLow.startsWith("bcd\n") && !qrCodeLow.startsWith("bcd\r\n")) {
			Logger.error("Falscher Qr-Code. Beginnt nicht mit: 'bcd[Zeilenumbruch]'");
			Logger.debug("Qr-Code: " + qrCode);
			throw new QrCodeHandlerException("Qr-Code not valid");
		}

		// Ist Hibiscus installiert?
		if (!Settings.isHibiscusInstalled()) {
			String text = "Der QR-Code enthlt eine SEPA-berweisung, aber Hibiscus ist nicht installiert.\n"
					+ "Bitte installieren sie das Hibiscus-Plugin.";
			Logger.warn(text);
			try {
				Application.getCallback().notifyUser(text);
			} catch (Exception e) {
				Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n"
						+ e.getMessage());
			}
		} else {

			String[] qrCodeFields = qrCode.split("\\r?\\n");
			ArrayList<String> qrFields = new ArrayList<String>(
					Arrays.asList(qrCodeFields));
			if (qrFields.size() < 7)
				throw new QrCodeHandlerException(
						"Qr-Code ist zu kurz. Muss mindestens 7 Element endhalten!.");

			// Auffuellen der List emit moeglich fehlenden Zeilen
			if (qrFields.size() < 12) {
				for (int i = qrFields.size(); i <= 11; i++) {
					qrFields.add("");
				}
			}

			BcdDataTransferObject bcdData = new BcdDataTransferObject();
			try {
				bcdData.setServiceKennung(qrFields.get(0));
				bcdData.setVersion(qrFields.get(1));
				bcdData.setKodierung(qrFields.get(2));
				bcdData.setFunktion(qrFields.get(3));
				bcdData.setBic(qrFields.get(4));
				bcdData.setEmpfaenger(qrFields.get(5));
				bcdData.setIban(qrFields.get(6));
				bcdData.setBetrag(qrFields.get(7));
				bcdData.setGeschaeftsCode(qrFields.get(8));
				bcdData.setReferenze(qrFields.get(9));
				bcdData.setVerwendungszweck(qrFields.get(10));
				bcdData.setBenutzerhinweis(qrFields.get(11));

				bcdData.oeffneNeueSEPA();

			} catch (BcdDataTransferObjcetDefinitionException bcde) {
				Logger.error("BCD Datenobjekt-Fehler: " + bcde.getMessage());
				throw new QrCodeHandlerException(bcde.getMessage());
			}
		}
	}

	@Override
	public boolean canHandle(String qrCode) {

		// Passt der QR-Code
		String qrCodeLow = qrCode.toLowerCase();
		if (qrCodeLow.startsWith("bcd\n001") || qrCodeLow.startsWith("bcd\r\n001")) {
			return true;
		}
		return false;
	}

	@Override
	public String getHandlerName() {
		return "BCD Vers. 001";
	}

	/* (nicht-Javadoc)
	 * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
	 */
	@Override
	public boolean isUeberweisung() {
		return true;
	}

}

/***********************************************************
 * History:
 * $Log BcdSepa_001_SCT.java,v$
 * $Date:: 2014-04-01 00:56:03$ Date of last commit
 * 
 * @Revision 1.3 Date: 04/07/2016
 *           berprfung anpassen auf Version 001, damit nicht auch Version 002
 *           erkannt wird.
 * @Revision 1.3 Date: 21/07/2014
 *           Auffllen des qrFiedls Array war zu kurz. Erhht auf 12 Felder.
 *            
 * @Revision 1.2 Date: 20.04.2014
 *           Throw BCD Datenobjekt-Fehler hinzugefgt, um in einem 
 *           Benutzer-Dialog darauf hinzuweisen.
 * 
 * @Revision 1.1 Date: 31.03.2014
 *           isUeberweisung hinzugefgt, um auch PDF-Dateien
 *           mit mehrere QR-Codes einlesen zu knnen
 * 
 *************************************************************/
