/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Map;

import org.apache.commons.lang.SystemUtils;

import de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers.BezahlCodeCommenFunctions;
import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.HibiscusAddress;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface
 * {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem
 * Hanlder fr Bezahlcode Kontak_V2 (SEPA)
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class BezahlcodeContactV2Handler implements IQrCodeHandler {

	/**
	 * Name des Handlers
	 */
	static final String HANDLER_NAME = "BezahlCode - Authority:contact and contact_v2";

	private static final String HANDLER_ID_1 = "bank://contact?";
	private static final String HANDLER_ID_2 = "bank://contact_v2?";

	static {
		QrCodeHandlerRegsistry.register(new BezahlcodeContactV2Handler());
	}

	@Override
	public void handleQrCode(String qrCode) throws QrCodeHandlerException {

		if ((!qrCode.toLowerCase().startsWith(HANDLER_ID_1))
				&& (!qrCode.toLowerCase().startsWith(
						HANDLER_ID_2))) {
			Logger.error("Falscher Qr-Code. Beginnt nicht mit: '"
					+ HANDLER_ID_1 + "' oder '"
					+ HANDLER_ID_2 + "'");
			Logger.debug("Qr-Code: " + qrCode);
			throw new QrCodeHandlerException("Qr-Code not valid");
		}
		if (!Settings.isHibiscusInstalled()) {
			String text = "Der QR-Code enthlt eine berweisung, aber Hibiscus ist nicht installiert.\n"
					+ "Bitte installieren sie das Hibiscus-Plugin.";
			Logger.warn(text);
			try {
				Application.getCallback().notifyUser(text);
			} catch (Exception e) {
				Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n"
						+ e.getMessage());
			}
		} else {
			try {
				String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1");
				String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode
						.indexOf("?") + 1);
				Map<String, String> params = BezahlCodeCommenFunctions
						.getQueryMap(qrCodeQuery);

				HibiscusAddress address = null;

				address = (HibiscusAddress) de.willuhn.jameica.hbci.Settings
						.getDBService().createObject(HibiscusAddress.class,
								null);

				// Name
				String value;
				if ((value = params.get("name")) != null) {
					if (value.equals(value.toUpperCase())) {
						
						try {
							if (Application.getCallback().askUser(
									BezahlCodeCommenFunctions.QUESTION_TEXT,
									new String[] { value } )) {
								value = BezahlCodeCommenFunctions.formatString(value);
							}
						} catch (Exception e) {}
					}
					address.setName(value);
				}
				if ((value = params.get("account")) != null) {
					address.setKontonummer(value);
				}
				if ((value = params.get("bnc")) != null) {
					address.setBlz(value);
				}
				if ((value = params.get("iban")) != null) {
					address.setIban(value);
				}
				if ((value = params.get("bic")) != null) {
					address.setBic(value);
				}
				if ((value = params.get("reason")) != null) {
					address.setKommentar(value);
				} else {
					//Test for reasonN
					String reasonN = "";
					if ((value = params.get("reason1")) != null) {
						reasonN = value;
					}
					if ((value = params.get("reason2")) != null) {
						reasonN += SystemUtils.LINE_SEPARATOR + value;
					}
					if ((value = params.get("reason3")) != null) {
						reasonN += SystemUtils.LINE_SEPARATOR + value;
					}
					if ((value = params.get("reason4")) != null) {
						reasonN += SystemUtils.LINE_SEPARATOR + value;
					}
					address.setKommentar(reasonN);
				}

				if (address != null) {
					// View neu zeichnen
					GUI.startView(
							de.willuhn.jameica.hbci.gui.views.EmpfaengerNew.class,
							address);
				}
			} catch (UnsupportedEncodingException e1) {
				throw new QrCodeHandlerException(
						"QR-Code konnte nicht valide dekodiert werden.");
			} catch (RemoteException e) {
				throw new QrCodeHandlerException(
						"Hibiscus Empfngersobject konnte nicht erstellt werden");
			}
		}
	}

	@Override
	public boolean canHandle(String qrCode) {
		if ((qrCode.toLowerCase().startsWith(HANDLER_ID_1))
				|| (qrCode.toLowerCase().startsWith(
						HANDLER_ID_2))) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public String getHandlerName() {
		return HANDLER_NAME;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
	 */
	@Override
	public boolean isUeberweisung() {
		return false;
	}

}

/***********************************************************
 * History:
 * $Log BezahlcodeSinglePaymentHandler.java.java,v$
 * $Date:: 2014-04-01 00:56:03$ Date of last commit
 * 
 * @Revision 1.0 Date: 16.12.2014
 * 
 *************************************************************/
