/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrcodehandler.BezahlcodeHandlers;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Map;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.Ueberweisung;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface
 * {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem
 * Hanlder fr Bezahlcode berweisungen (nicht SEPA)
 * 
 * @author munstfr
 * @version 1.2
 * 
 */
public class BezahlcodeSinglePaymentHandler implements IQrCodeHandler {

	/**
	 * Name des Handlers
	 */
	static final String HANDLER_NAME = "BezahlCode - Authority:singelpayment";

	private static final String HANDLER_ID = "bank://singlepayment?";

	static {
		QrCodeHandlerRegsistry.register(new BezahlcodeSinglePaymentHandler());
	}

	@Override
	public void handleQrCode(String qrCode) throws QrCodeHandlerException {

		if (!qrCode.toLowerCase().startsWith(HANDLER_ID)) {
			Logger.error("Falscher Qr-Code. Beginnt nicht mit: '"
					+ HANDLER_ID + "'");
			Logger.debug("Qr-Code: " + qrCode);
			throw new QrCodeHandlerException("Qr-Code not valid");
		}
		if (!Settings.isHibiscusInstalled()) {
			String text = "Der QR-Code enthlt eine berweisung, aber Hibiscus ist nicht installiert.\n"
					+ "Bitte installieren sie das Hibiscus-Plugin.";
			Logger.warn(text);
			try {
				Application.getCallback().notifyUser(text);
			} catch (Exception e) {
				Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n"
						+ e.getMessage());
			}
		} else {
			try {
				String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1");
				String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode
						.indexOf("?") + 1);
				Map<String, String> params = BezahlCodeCommenFunctions
						.getQueryMap(qrCodeQuery);

				Ueberweisung u = null;

				u = (Ueberweisung) de.willuhn.jameica.hbci.Settings
						.getDBService().createObject(Ueberweisung.class, null);
				// Name
				String value;
				if ((value = params.get("name")) != null) {
					if (value.equals(value.toUpperCase())) {

						try {
							if (Application.getCallback().askUser(
									BezahlCodeCommenFunctions.QUESTION_TEXT,
									new String[] { value })) {
								value = BezahlCodeCommenFunctions
										.formatString(value);
							}
						} catch (Exception e) {
						}
					}
					u.setGegenkontoName(value);
				}
				if ((value = params.get("account")) != null) {
					u.setGegenkontoNummer(value);
				}
				if ((value = params.get("bnc")) != null) {
					u.setGegenkontoBLZ(value);
				}
				if ((value = params.get("amount")) != null) {
					u.setBetrag(Double.valueOf(value.replace(",", ".")));
				}
				if ((value = params.get("reason")) != null) {
					u.setZweck(value);
				} else {
					String zweck = "";
					if ((value = params.get("reason1")) != null) {
						zweck += value + " ";
					}
					if ((value = params.get("reason2")) != null) {
						zweck += value + " ";
					}
					if ((value = params.get("reason3")) != null) {
						zweck += value + " ";
					}
					if ((value = params.get("reason4")) != null) {
						zweck += value;
					}
					u.setZweck(zweck);
				}
				if ((value = params.get("executiondate")) != null) {
					Date termin = new Date(Long.valueOf(value));
					u.setTermin(termin);
				}
				if (u != null) {
					// View neu zeichnen
					GUI.startView(
							de.willuhn.jameica.hbci.gui.views.UeberweisungNew.class,
							u);
				}
			} catch (UnsupportedEncodingException e1) {
				throw new QrCodeHandlerException(
						"QR-Code konnte nicht valide dekodiert werden.");
			} catch (RemoteException e) {
				throw new QrCodeHandlerException(
						"Hibiscus berweisungsobject konnte nicht erstellt werden");
			}
		}
	}

	@Override
	public boolean canHandle(String qrCode) {
		return qrCode.toLowerCase().startsWith(HANDLER_ID);
	}

	@Override
	public String getHandlerName() {
		return HANDLER_NAME;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
	 */
	@Override
	public boolean isUeberweisung() {
		return true;
	}

}

/***********************************************************
 * History: $Log BezahlcodeSinglePaymentHandler.java.java,v$ $Date:: 2014-04-01
 * 00:56:03$ Date of last commit
 * 
 * @Revision 1.1 Date: 31.03.2014 isUeberweisung hinzugefgt, um auch
 *           PDF-Dateien mit mehrere QR-Codes einlesen zu knnen
 * 
 *           1.2 Date: 20.12.2014 - getQueryMap ausgelagert -
 *           HandlerIdentificationString hinzugefgt - Name wird auf
 *           Grobuchstaben untersucht
 * 
 *************************************************************/
