/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner;

import de.willuhn.jameica.system.Application;
import de.willuhn.util.I18N;

/**
 * Settings Klasse fr den jameica.qrscanner Basiert auf den Settings-Klasse aus
 * dem Beispiel-Plugin von Olaf Willuhn.
 * 
 * @author willuhn/munstfr
 * @version 1.1
 * @see <a
 *      href="http://www.willuhn.de/products/jameica/dev.php">http://www.willuhn.de/products/jameica/dev.php</a>
 * 
 * 
 */
public class Settings {

	/**
	 * System settings von Jameica
	 */
	private static de.willuhn.jameica.system.Settings jameicaSettings = Application
			.getPluginLoader().getPlugin(QrCodeScanner.class).getResources()
			.getSettings();
	
	/**
	 * I18N local Variable
	 */
	private static I18N i18n;
	/**
	 * Is Hibiscus plugin also installed
	 */
	private static Boolean hibiscusInstalled;

	/**
	 * Small helper function to get the translator.
	 * 
	 * @return translator.
	 */
	public static I18N i18n() {
		if (i18n != null)
			return i18n;
		i18n = Application.getPluginLoader().getPlugin(QrCodeScanner.class)
				.getResources().getI18N();
		return i18n;
	}

	/**
	 * berprft, ob das Hibiscus -Plugin installiert ist.
	 * 
	 * @return isHibiscusInstalled <b>true</b>, wenn Hibiscus installiert ist,
	 *         <b>false</b> sonst
	 */
	public static Boolean isHibiscusInstalled() {
		if (hibiscusInstalled != null) 
			return hibiscusInstalled;
		

		String hibiscusClassName = "de.willuhn.jameica.hbci.HBCI";
		hibiscusInstalled = Application.getPluginLoader().isInstalled(
				hibiscusClassName);

		return hibiscusInstalled;
	}

	/**
	 * Soll IpStreaming Webcam verwendet werden. 
	 * 
	 * @param useIpStreamDriver
	 */
	public static void setUseIpStreamDriver(final Boolean useIpStreamDriver) {
		jameicaSettings.setAttribute("qrcode.useIpStreamDriver", useIpStreamDriver);
	}

	/**
	 * Liefert zur&uuml;ck, ob die IpStreaming Webcam verwendet werden soll.
	 * 
	 * @return verwende IpStreamining Webcam
	 */
	public static boolean isUseIpStreamDriver() {
		return jameicaSettings.getBoolean("qrcode.useIpStreamDriver", false);
	}

	/**
	 * Liefert zur&uuml;ck, ob DroidCam App als Webcam verwendet werden soll.
	 * 
	 * @return verwende DroidCam App als Webcam
	 */
	public static boolean isUseDroidCam() {
		return jameicaSettings.getBoolean("qrcode.useDroidCam", false);
	}
	
	/**
	 * Setzt, ob DroidCam App als Webcam verwendet werden soll.
	 * @param useDroidCam benutzte DroidCam.
	 */
	public static void setUseDroidCam(final Boolean useDroidCam ) {
		 jameicaSettings.setAttribute("qrcode.useDroidCam", useDroidCam);
	}

	/**
	 * Liefert den DroidCam Host. 
	 * @return den DroidCam Host.
	 */
	public static String getDroidCamHost() {
		return jameicaSettings.getString("qrcode.droidCamHost", "");
	}
	
	/**
	 * Setzt den DroidCam Host. 
	 * @return den DroidCam Host.
	 */
	public static void setDroidCamHost(String droidCamHost) {
		jameicaSettings.setAttribute("qrcode.droidCamHost", droidCamHost);
	}

	public static Integer getDroidCamPort() {
		return jameicaSettings.getInt("qrcode.droidCamPort", 4747);
	}
	
	/**
	 * Setzt den DroidCam Port. 
	 * @return den DroidCam port.
	 */
	public static void setDroidCamPort(int droidCamPort) {
		jameicaSettings.setAttribute("qrcode.droidCamPort", droidCamPort);
	}
}


/***********************************************************
 * History:
 * $Log Settings.java,v$
 * $Date:: 2014-04-01 00:56:03$ Date of last commit
 * 
 * @Revision 1.1 Date: 11.07.2016
 *           qrcode.useIpStreamDriver hinzugefgt
 * 
 *************************************************************/
