/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner;

import java.awt.Dimension;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.jmdns.JmDNS;
import javax.jmdns.ServiceEvent;
import javax.jmdns.ServiceInfo;
import javax.jmdns.ServiceListener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamCompositeDriver;
import com.github.sarxos.webcam.WebcamDiscoveryEvent;
import com.github.sarxos.webcam.WebcamDiscoveryListener;
import com.github.sarxos.webcam.WebcamException;
import com.github.sarxos.webcam.ds.buildin.WebcamDefaultDriver;
import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;
import com.github.sarxos.webcam.ds.ipcam.IpCamDriver;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.frankmuenster.jameica.qrscanner.ipcam.DroidCamDevice;
import de.willuhn.logging.Logger;

/**
 * Webcam berwachung implementiert das
 * {@link com.github.sarxos.webcam.WebcamDiscoveryListener} Interface. Es
 * ermittelt die max. Auflsung einer neuen gefunden Webcam.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class WebcamWatchdog implements WebcamDiscoveryListener, ServiceListener {

	private static List<MyWebcam> webcams = new ArrayList<>();

	private static JmDNS jmdns = null;
	private static MyCompositeDriver driver = new MyCompositeDriver();

	/**
	 * Customized webcam driver.
	 */
	public static class MyCompositeDriver extends WebcamCompositeDriver {

		public MyCompositeDriver() {
			add(new WebcamDefaultDriver());
			add(new IpCamDriver());
		}
	}

	// register custom composite driver
	static {
		// Webcam.setHandleTermSignal(true);
		Webcam.setDriver(driver);

	}

	public static MyCompositeDriver getDriver() {
		return driver;
	}

	/**
	 * @return initialised
	 */
	public static Boolean isInitiaised(Webcam webcam) throws WebcamException {
		for (MyWebcam myWebCam : webcams) {
			if (myWebCam.getWebcamName().equals(webcam.getName())) {
				return myWebCam.initialised.get();
			}
		}
		throw new WebcamException("Webcam nicht gefunden");
	}

	/**
	 * Registriert die Klasse im WebcamDiscoveryListener und initialisiert die
	 * Webcam Klasse.
	 */
	public WebcamWatchdog() {

		// Listener anhngen
		// Webcam.addDiscoveryListener(this);
		Webcam.getDefault();

		// JmDNS initialisieren
		if (jmdns == null) {

			try {
				jmdns = JmDNS.create(InetAddress.getLocalHost());
				// Mini Webcam App(iphone)
				jmdns.addServiceListener("_mini_webcam_101._tcp.local.", this);
				jmdns.addServiceListener("_aircamera._tcp.local.", this);

			} catch (UnknownHostException e) {
				System.out.println(e.getMessage());
			} catch (IOException e) {
				System.out.println(e.getMessage());
			}
		}

		// DroidCam
		if (Settings.isUseDroidCam()) {
			Logger.debug("Adding DroidCam device: "
					+ Settings.getDroidCamHost() + ":"
					+ Settings.getDroidCamPort());
			DroidCamDevice ipCam = new DroidCamDevice(
					Settings.getDroidCamHost(), Settings.getDroidCamPort());
			if (ipCam != null) {
				IpCamDeviceRegistry.register(ipCam);
			}
		}

	}

	public static List<String> getWebcams() {

		List<String> wcams = new ArrayList<>();

		for (MyWebcam mw : webcams) {
			wcams.add(mw.getWebcamName());
		}

		return wcams;

	}

	@Override
	public void webcamFound(WebcamDiscoveryEvent event) {
		Logger.debug("Watchdog - Webcam has been connected: "
				+ event.getWebcam().getName());

		// add to list
		MyWebcam webcam = new MyWebcam(event.getWebcam().getName(), event
				.getWebcam().getDriver().getClass());
		webcams.add(webcam);

		for (MyWebcam element : webcams) {
			Logger.debug("  " + element.getWebcamName() + " Class: "
					+ element.getWebcamDriver());
		}

	}

	@Override
	public void webcamGone(WebcamDiscoveryEvent event) {
		Logger.debug("Watchdog - Webcam has been disconnected: "
				+ event.getWebcam().getName());
		Webcam removedWebcam = event.getWebcam();
		for (Iterator<MyWebcam> myWebCam = webcams.iterator(); myWebCam
				.hasNext();) {
			MyWebcam mw = myWebCam.next();
			if (mw.getWebcamName().equals(removedWebcam.getName())) {
				myWebCam.remove();
				Logger.debug("Watchdog - Webcam removed from list");
			}
		}

		for (MyWebcam element : webcams) {
			Logger.debug("  " + element.getWebcamName() + " Class: "
					+ element.getWebcamDriver());
		}

	}

	/**
	 * @see javax.jmdns.ServiceListener#serviceAdded(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceAdded(ServiceEvent event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("Service added: " + event.getInfo());

	}

	/**
	 * @see javax.jmdns.ServiceListener#serviceRemoved(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceRemoved(ServiceEvent event) {

		ServiceInfo serviceInfo = event.getInfo();

		Logger.debug("Watchdog - IpWebcam has been disconnected: "
				+ serviceInfo.getName());

		String webcamName = serviceInfo.getName();

		if (IpCamDeviceRegistry.isRegistered(webcamName)) {
			IpCamDeviceRegistry.unregister(webcamName);
		}
	}

	/**
	 * @see javax.jmdns.ServiceListener#serviceResolved(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceResolved(ServiceEvent event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("Service resolved: " + event.getInfo());
		Logger.debug("Service application: " + event.getInfo().getApplication());
		Logger.debug("Service url: " + event.getInfo().getURL());

		ServiceInfo serviceInfo = event.getInfo();

		try {
			String name = serviceInfo.getName();
			String url = serviceInfo.getURL();
			IpCamDevice ipCamDevice = new IpCamDevice(name, url, IpCamMode.PUSH);
			if (!IpCamDeviceRegistry.isRegistered(ipCamDevice)) {
				IpCamDeviceRegistry.register(ipCamDevice);
			}

		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private static final class MyWebcam {

		private String webcamName;
		private Class webcamDriver;
		private AtomicBoolean initialised = new AtomicBoolean(true);

		/**
		 * @param webcam
		 */
		public MyWebcam(String webcamName, Class webcamDriver) {
			this.webcamName = webcamName;
			this.webcamDriver = webcamDriver;
			// get and set Webcam resolution
			// getWebcamInfo();

		}

		public String getWebcamName() {
			return this.webcamName;
		}

		public Class getWebcamDriver() {
			return this.webcamDriver;
		}

		/**
		 * @return the initialised
		 */
		protected boolean isInitialised() {
			return initialised.get();
		}

		/**
		 * @param initialised
		 *            the initialised to set
		 */
		protected void setInitialised(boolean initialised) {
			this.initialised.set(initialised);
		}

		private void getWebcamInfo() {

			Thread getInfo = new Thread() {
				@Override
				public void run() {

					setName("WebcamInfo - " + webcamName);

					Webcam.setDriver(webcamDriver);
					Webcam webcam = Webcam.getWebcamByName(webcamName);

					Dimension[] nonStandardResolutions = new Dimension[] { new Dimension(
							20000, 10000), };
					try {
						// your camera have to support HD720p to run this code
						webcam.getLock().disable();
						webcam.setCustomViewSizes(nonStandardResolutions);
						webcam.setViewSize(new Dimension(20000, 10000));
						webcam.open();
						webcam.close();
						Logger.info("Maximale Webcam Auflsung: "
								+ webcam.getViewSize().toString());

					} catch (Exception e) {
						Logger.error("Could not open/close webcam to enumerta max. resolution"
								+ webcam.getName());
					}
					if (!isInitialised())
						setInitialised(true);
				}
			};
			getInfo.start();
		}

	}

}
