/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.action;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.dialog.QrScannerDialog;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * View und Kontroller zum Scannen und weiter bearbeiten eines QR-Codes.
 * 
 * @author munstfr
 * @version 1.1
 * 
 */
public class QrScannerAction implements Action {

	/**
	 * ffnet einen QrScannerDialog und gibt das Ergebnis an einen passenden
	 * QR-Code Handler weiter, falls mglich.
	 */
	@Override
	public void handleAction(Object context) throws ApplicationException {

		String qrText = null;
		// show dialog and start scanning
		try {

			QrScannerDialog d = new QrScannerDialog();
			qrText = (String) d.open();

			if (qrText != null) {
				Logger.debug("Scann Result:");
				Logger.debug(qrText);

				IQrCodeHandler handler = QrCodeHandlerRegsistry
						.getQrCodeHandler(qrText);
				if (handler != null) {
					try {
						handler.handleQrCode(qrText);
					} catch (QrCodeHandlerException qre) {
						Logger.error("Qr-Code handler error: "
								+ qre.getMessage());
						String niceText = "Der QR-Code konnte nicht vollstndig bearbeitet werden.\n\n"
								+ "Bitte berprfen Sie die System-Logs fr weitere Informationen.\n";
						try {
							Application.getCallback().notifyUser(niceText);
						} catch (Exception e) {
							// Hier brauchen wir nichts zu tun!
							e.printStackTrace();
						}

					}
				} else {
					Logger.info(Settings.i18n().tr(
							"No matching qr-code handler found"));
					String niceText = "Fr den gefundenen Qr-Code (siehe debug log) wurde \n"
							+ "kein passender Handler zur weiteren Verabeitung\n"
							+ "gefunden.\n\nBitte berprfen Sie den Qr-Code.";
					Application.getCallback().notifyUser(niceText);
				}

			} else {
				Logger.debug("Kein QR String gefunden, oder Dialog mittels X geschlossen");
			}

		} catch (OperationCanceledException oce) {
			Logger.info("Einlesen des QR Codes vom Benutzer abgebrochen");
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		}

		catch (Exception e) {
			Logger.error("Unexpected error: " + e.getMessage());
		}
	}
}

/***********************************************************
 * History:
 * $Log QrScannerAction.java,v$
 * $Date:: 2014-04-01 00:56:03$ Date of last commit
 * 
 * @Revision 1.1 Date: 20.04.2014
 *           Benutzer-Dialog hinzugefgt, um auf fehlerhafte
 *           QR-Code-Verarbeitung (Abbruch mit Error) hinzuweisen.
 * 
 * 
 *************************************************************/
