/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */


package de.frankmuenster.jameica.qrscanner.gui.controller;

import java.io.File;
import java.io.FileInputStream;

import de.frankmuenster.jameica.qrscanner.QrCodeScanner;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.io.FileFinder;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.util.InfoReader;
import de.willuhn.logging.Logger;
import de.willuhn.util.I18N;

/**
 * Controller fuer den Lizenzinformationen Dialog
 * 
 * Basiert auf dem {@link de.willuhn.jameica.gui.internal.controller#LincenseControl}
 * 
 * @author willuhn/munstfr
 * @version 1.0
 * 
 */
public class LicenseControl extends AbstractControl {

	/**
	 * FormTextPart um dem Text schn formatieren zu knnen 
	 */
	private FormTextPart libList = null;
	/**
	 * i18n bersetzungs-Objekt aus unseren Settings
	 */
	private static I18N i18n = Settings.i18n();

	/**
	 * Standrdart Konstruktor liefert ein LincenceControl object 
	 * 
	 * @param view die View, fuer die dieser LicenceControl zustaendig ist
	 */
	public LicenseControl(AbstractView view) {
		super(view);
	}

	/**
	 * Liefert eine Liste mit allen direkt von jameica.qrscanner verwendeten Komponenten.
	 * 
	 * @return Liste der Komponenten.
	 */
	public Part getLibList() {
		if (libList != null)
			return libList;

		AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				QrCodeScanner.class);

		StringBuffer buffer = new StringBuffer();
		buffer.append("<form>");

		Manifest manifest = null;
		try {
			manifest = plugin.getManifest();
		} catch (Exception e) {
			Logger.error("unable to read info.xml from plugin jameica.qrscanner", e);
		}
		buffer.append("<p><span color=\"header\" font=\"header\">"
				+ i18n.tr("QR-Code Scanner") + "</span></p>");
		if (manifest != null) {
			buffer.append("<p>");
			buffer.append(manifest.getDescription());
			buffer.append("<br/>" + manifest.getHomepage());
			buffer.append("<br/>" + manifest.getLicense());
			buffer.append("</p>");
		}

		String path = plugin.getManifest().getPluginDir();

		FileFinder finder = new FileFinder(new File(path + "/lib"));
		finder.matches(".*?info\\.xml$");
		File[] infos = finder.findRecursive();
		for (int i = 0; i < infos.length; ++i) {
			if (!infos[i].isFile() || !infos[i].canRead()) {
				Logger.warn("unable to read " + infos[i] + ", skipping");
				continue;
			}

			try {
				InfoReader ir = new InfoReader(new FileInputStream(infos[i]));
				buffer.append("<p>");
				buffer.append("<b>" + ir.getName() + "</b>");
				buffer.append("<br/>" + ir.getDescription());
				buffer.append("<br/>" + ir.getUrl());
				buffer.append("<br/>" + ir.getLicense());
				buffer.append("</p>");
			} catch (Exception e) {
				Logger.error("unable to parse " + infos[0], e);
			}
		}
		buffer.append("</form>");

		libList = new FormTextPart(buffer.toString());
		return libList;
	}
}
