package de.frankmuenster.jameica.qrscanner.gui.controller;

import java.net.MalformedURLException;
import java.util.List;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.WebcamWatchdog;
import de.frankmuenster.jameica.qrscanner.gui.views.SettingsView;
import de.frankmuenster.jameica.qrscanner.ipcam.DroidCamDevice;
import de.frankmuenster.jameica.qrscanner.utils.Validators;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.IntegerInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontroller fr die Settings View
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SettingsController extends AbstractControl {

	SettingsView view;

	private CheckboxInput useDroidCam = null;
	private TextInput droidCamHost = null;
	private IntegerInput droidCamPort = null;
	private TextAreaInput discoverdWebcams;

	private Button refreshBtn;

	/**
	 * Erstellt eine neue Instanz des Kontrollers
	 * 
	 * @param view
	 * @throws MalformedURLException
	 */
	public SettingsController(SettingsView view) {
		super(view);
		this.view = view;

	}

	/**
	 * 
	 * @return Ein CheckBoxInput zur Auswahl, ob die DroidCam App als Wecam
	 *         benutzt werden soll.
	 */
	public CheckboxInput getUseDroidCam() {
		if (useDroidCam != null)
			return useDroidCam;

		Logger.debug("UseDroidCam: " + Settings.isUseDroidCam());
		useDroidCam = new CheckboxInput(Settings.isUseDroidCam());
		useDroidCam.addListener(new Listener() {

			@Override
			public void handleEvent(Event event) {
				handleUseDroidCamEvent(event);
			}
		});

		return useDroidCam;
	}

	/**
	 * Liefert ein Texteingabefeld f&uuml;r den DroidCam Host
	 * 
	 * @return TextInput f&uuml;r DroidCam IP
	 */
	public TextInput getDroidCamHost() {
		if (droidCamHost != null)
			return droidCamHost;

		droidCamHost = new TextInput(Settings.getDroidCamHost(), 15,
				"192.178.0.200");
		return droidCamHost;
	}

	/**
	 * Liefert ein DecimalInput f&uuml;r den DroidCam Port.
	 * 
	 * @return TextInput f&uuml;r DroidCam Port
	 */
	public IntegerInput getDroidCamPort() {
		if (droidCamPort != null)
			return droidCamPort;

		droidCamPort = new IntegerInput(Settings.getDroidCamPort());
		droidCamPort.setComment("");
		return droidCamPort;
	}

	public TextAreaInput getDiscoverdWebcams() {

		if (discoverdWebcams != null)
			return discoverdWebcams;

		discoverdWebcams = new TextAreaInput(Settings.i18n().tr("Nothing found"));
		discoverdWebcams.setHeight(130);
		discoverdWebcams.setComment("");

		List<Webcam> webcams = Webcam.getWebcams();

		StringBuffer webcamNames = new StringBuffer();
		for (Webcam webcam : webcams) {
			webcamNames.append(webcam.getName() + "\n");
		}
		discoverdWebcams.setValue(webcamNames.toString());

		return discoverdWebcams;

	}

	public Button getRefeshBtn() {
		if (refreshBtn != null)
			return refreshBtn;

		refreshBtn = new Button(Settings.i18n().tr("Refresh"), new Action() {

			@Override
			public void handleAction(Object context)
					throws ApplicationException {
				Webcam.resetDriver();
				Webcam.setDriver(WebcamWatchdog.getDriver());
				Webcam.getDefault();

				StringBuffer webcamNames = new StringBuffer();
				List<WebcamDevice> dev = Webcam.getDriver().getDevices();
				for (WebcamDevice webcamDevice : dev) {
					Logger.debug("Dev: " + webcamDevice.getName());
					webcamNames.append(webcamDevice.getName() + "\n");
				}
				discoverdWebcams.setValue(webcamNames.toString());
				discoverdWebcams.getControl().redraw();
			}
		});

		return refreshBtn;

	}

	/**
	 * Speichert die aktuellen Einstellungen im Settings Objekt.
	 */
	public void speichern() {

		// Check DroidCam IP if enabled
		String ip = (String) droidCamHost.getValue();
		if ((boolean) useDroidCam.getValue()
				&& !Validators.isValidInet4Address(ip)) {
			GUI.getStatusBar().setErrorText(
					"DroidCam IP Adresse ist nicht korret formatiert.");

		} else {
			Settings.setUseDroidCam((Boolean) useDroidCam.getValue());
			Settings.setDroidCamHost((String) droidCamHost.getValue());
			Settings.setDroidCamPort((int) droidCamPort.getValue());
			GUI.getStatusBar().setSuccessText("Einstellungen gespeichert.");

			updateCamList();
		}

	}

	private void updateCamList() {

		if (IpCamDeviceRegistry.isRegistered("DroidCam")) {
			// ipCam.dispose();
			Logger.debug("Unregister : "
					+ IpCamDeviceRegistry.unregister("DroidCam"));

		}
		if ((Boolean) useDroidCam.getValue()) {
			DroidCamDevice ipCam = new DroidCamDevice(
					(String) droidCamHost.getValue(),
					(int) droidCamPort.getValue());
			if (ipCam != null) {
				IpCamDeviceRegistry.register(ipCam);
			}
		}

		StringBuffer webcamNames = new StringBuffer();
		List<WebcamDevice> dev = Webcam.getDriver().getDevices();
		for (WebcamDevice webcamDevice : dev) {
			Logger.debug("Dev: " + webcamDevice.getName());
			// Logger.debug("Dev: " + webcamDevice.;
			webcamNames.append(webcamDevice.getName() + "\n");
		}
		discoverdWebcams.setValue(webcamNames.toString());
		discoverdWebcams.getControl().update();
	}

	/**
	 * @param event
	 */
	protected void handleUseDroidCamEvent(Event event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("useDroidCam Event" + event.toString());

		if ((boolean) useDroidCam.getValue()) {
			Logger.debug("Enable droidCam Fields");
			droidCamHost.enable();
			droidCamPort.enable();
		} else {
			Logger.debug("Disable droidCam Fields");
			droidCamHost.disable();
			droidCamPort.disable();
		}

	}

}
