/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.qrscanner.QrCodeScanner;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.LabelGroup;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * "AboutDialog..." Dialog. Basiert auf dem About dialog aus Jameica von Olaf Willuhn
 * 
 * @author willuhn/munstfr
 * @version 1.2
 * 
 */
public class AboutDialog extends AbstractDialog<Object> {

	/**
	 * Liefetr einen modalen Dialog, der mittel .open() angezeigt werden kann
	 * 
	 * @param position
	 *            die Position, an der der Dialog angezeigt werden soll.
	 *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
	 */
	public AboutDialog(final int position) {

		super(position);

		final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				QrCodeScanner.class);

		this.setTitle(Settings.i18n().tr("About..."));
		this.setPanelText(Settings.i18n().tr("QR-Code Scanner {0}",
				plugin.getManifest().getVersion().toString()));
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected Object getData() throws Exception { // NOPMD
		return null;
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) throws Exception { // NOPMD

		final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				QrCodeScanner.class);

		final Label label = GUI.getStyleFactory().createLabel(parent, SWT.BORDER);
		label.setImage(SWTUtil.getImage("qrscanner.jpg"));

		final Container container = new LabelGroup(parent, Settings.i18n().tr(
				"Version info"), true);

		final FormTextPart text = new FormTextPart();
		text.setText("<form>"
				+ "<p><b>QR-Code Scanner - Plugin fr Jameica</b></p>"
				+ "<p>Lizenz: EUPL [<a href=\"http://ec.europa.eu/idabc/eupl\">http://ec.europa.eu/idabc/eupl</a>]<br/>"
				+ "Copyright by Frank Mnster [<a href=\"mailto:jameica@frankmuenster.de\">jameica@frankmuenster.de</a>]<br/>"
				+ "<a href=\"http://progs.frankmuenster.de/jameica-qr-code-scanner/\">http://progs.frankmuenster.de/jameica-qr-code-scanner</a></p>"
				+ "<p>Software-Version: " + plugin.getManifest().getVersion()
				+ "<br/>" + "Build: " + plugin.getManifest().getBuildnumber()
				+ " [Datum " + plugin.getManifest().getBuildDate() + "]</p>"
				+ "</form>");
		container.addPart(text);

		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(Settings.i18n().tr("Close"), new Action() {
			/**
			 * 
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				close();
			}
		}, null, true, "window-close.png");

		container.addButtonArea(buttons);

		setSize(SWT.DEFAULT, 525);

	}

}

/***********************************************************
 * History:
 * $Log ImageLabelPart.java,v$
 * $Date:: 2014-05-07 13:10:43#$ Date of last commit
 * 
 * Revision 1.2 Date: 22.09.2016
 *     Hhe um 50 px verkleinert
 *     Webcam Info entfernt
 *      
 * Revision 1.1 Date: 27.04.2014
 *     Hhe um 25 px vergrert
 *      
 *********************************************************/
