/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.dialog;

import javax.swing.SwingUtilities;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.github.sarxos.webcam.Webcam;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.QrCodeFoundListener;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.QrCodeScannerThread;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.WebcamSelectedListener;
import de.frankmuenster.jameica.qrscanner.gui.parts.FileImageDropPastePart;
import de.frankmuenster.jameica.qrscanner.gui.parts.ImageLabelPart;
import de.frankmuenster.jameica.qrscanner.gui.parts.WebcamSelector;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.LabelInput;
import de.willuhn.jameica.gui.input.MultiInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * 
 * Klasse die einen modalen Dialog erstellt und einen Thread startet, in dem
 * Bilder der default Webcam angezeigt und nach QRCodes gescannt werden. <br/>
 * In Version 1.1 ist das Droppen/ffnen von PDF Dateien hinzugekommen. In
 * Version 1.3 die Mglichkeit eine Bild per Zwischenablage zu scannen. Der
 * Dialog schliet sich automatisch, wenn ein QR Code gefunden wurde. <br/>
 * Das Scannen kann mit Abbrechen unterbrochen werden
 * 
 * @author munstfr
 * @version 1.5
 * 
 */
public class QrScannerDialog extends AbstractDialog<String> implements
		QrCodeFoundListener, WebcamSelectedListener {

	/**
	 * i18n bersetzer aus unseren Settings
	 */
	private final static I18N I18N = Settings.i18n();
	/**
	 * Enthlt das Scan-Ergebnis, also den QR-Code als String, oder eben nix
	 */
	private String result = null;
	/**
	 * Ein Webcam Selector Panel
	 */
	private WebcamSelector webcamSelector;

	/**
	 * Ein Label, um Webcam Infos anzuzeigen
	 */
	private transient TextAreaInput webcamInfo;
	/**
	 * Das ImageLabelPart Part, in dem das Webcam Bild dargestellt werden soll
	 */
	private ImageLabelPart imgLabel;
	/**
	 * der Hintergrund Thread indem die Webcam stndig nach neuen Bilder
	 * abgefragt und diese nach QR-Codes gescannt werden
	 */
	private QrCodeScannerThread scanTask = null;

	/**
	 * Das Part zum droppen/ffnen von PDF Dateien
	 */
	private FileImageDropPastePart fileImageDropLabel = null;

	/**
	 * Standard Konstruktor<br/>
	 * Setzt Position, Gre und Titel
	 * 
	 */
	public QrScannerDialog() {

		super(AbstractDialog.POSITION_CENTER, false);
		this.setTitle(I18N.tr("Scanning qr-code"));
		this.setSize(330, 550);

	}

	@Override
	protected void paint(Composite parent) throws Exception {

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		// layout.makeColumnsEqualWidth = true;
		parent.setLayout(layout);

		// Erstelle einfachen Container
		Container container = new SimpleContainer(parent, true, 1);

		// Webcam-Auswahlbox
		webcamSelector = new WebcamSelector();
		webcamSelector.addListener(this);
		container.addPart(webcamSelector);
		container.addSeparator();

		Webcam webcam = Webcam.getWebcamByName(webcamSelector.getValue());

		webcamInfo = new TextAreaInput("\n\n\n");
		webcamInfo.setHeight(60);
		container.addPart(webcamInfo);
		container.addSeparator();
		String text = "";

		// Ist berhaupt ein Webcam angeschlossen und verfgbar
		if (webcam == null) {
			text = "Keine Webcam verbunden oder die Webcam wird "
					+ "leider nicht untersttzt!\n "
					+ "Die Funktionen zum Einlesen von PDF-Dateien "
					+ "stehen natrlich zur Verfgung.";
		} else {

			// Erstelle schnes ImageLable in anstndiger Gre
			Point size = new Point(320, 240);
			imgLabel = new ImageLabelPart(size);
			container.addPart(imgLabel);
			container.addSeparator();
		}
		webcamInfo.setValue(text);

		
		// drop target
		fileImageDropLabel = new FileImageDropPastePart();
		fileImageDropLabel.addListener(this);
		container.addPart(fileImageDropLabel);

		// Abbrechen Button einfgen
		Button cancle = new Button(I18N.tr("Cancel"), new Action() {
			public void handleAction(Object context)
					throws ApplicationException {
				cancleTask();
				throw new OperationCanceledException();
			}
		}, null, false, "process-stop.png");
		ButtonArea buttons = new ButtonArea();
		buttons.addButton(cancle);
		container.addButtonArea(buttons);

		//String webcamName = webcamSelector.getValue();
		if (webcam != null) {
			// Scan-Task starten
			try {
				doScanning();
			} catch (Exception e) {
				text = I18N.tr("Could not start scanning task.");
				text += " " + I18N.tr("Please close the dialog and try again.");
				imgLabel.setText(text);
				Logger.error(I18N.tr("Could not start scanning task."));
			}
		}
	}

	@Override
	protected String getData() throws Exception {
		/*
		 * berprfe, ob der scanTask noch luft. Das kann passieren, wenn der
		 * Benutzer den Dialog mittels des X schliet.
		 */
		if ((scanTask != null) && (!scanTask.isInterrupted())) {
			cancleTask();
		}
		return this.result;
	}

	/**
	 * Scan-Task unterbrechen
	 */
	private void cancleTask() {
		if (scanTask != null) {
			scanTask.removeListener(this);
			scanTask.interrupt();
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
			}
		}

	}

	/**
	 * Generiert und implementiert den BackgroundTask zur Webcambildabfrage und
	 * QR-Code scannen.
	 * 
	 * @throws Exception
	 *             wenn Scanner-Task nicht erfolgreich gestartet werden konnte.
	 */
	private void doScanning() throws Exception {

		final String webcamName = webcamSelector.getValue();

		scanTask = new QrCodeScannerThread(imgLabel, webcamInfo, webcamName);
		scanTask.addListener(this);
		SwingUtilities.invokeLater(scanTask);

	}

	/**
	 * Listener for QR-Code gefunden. Im Fehlerfall is das <b>result</b> gleich
	 * <b>null</b>
	 */
	@Override
	public void notifyOfQrCodeFound(String result) {
		// Ergebnis sichern
		this.result = result;
		// wichtig Dialog schlieen
		close();
	}

	@Override
	public void notifyOfWebcamsChanged() {

		String webcamName = webcamSelector.getValue();
		Logger.debug("Neue Webcam ausgewhlt: " + webcamName);
		cancleTask();
		while (scanTask.isAlive()) {
			try {
				Logger.debug("ScanTask still alive");
				Thread.sleep(500);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		try {
			doScanning();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}

/***********************************************************
 * History: $Log QrScannerDialog.java,v$ $Date:: 2014-04-01 00:56:03$ Date of
 * last commit
 * 
 * @Revision 1.2 Date: 08.07.2016 MANTISBT-9: Untersttzung mehrere Webcams
 *           hinzugefgt
 * @Revision 1.4 Date: 19.06.2014 Bug 788, scanTask ist null, wenn keine Webcam
 *           angeschlossen ist
 * @Revision 1.3 Date: 27.04.2014 Paste Button hinzugefgt
 * @Revision 1.2 Date: 20.04.2014 Dialoggre und Text angepasst, falls keine
 *           Webcam gefunden wurde
 * @Revision 1.1 Date: 2014-04-01 Drop&ffnen von PDF Dateien hinzugefgt
 * 
 *********************************************************/
