/*
 * Copyright 2014 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.qrscanner.gui.navigation;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.datasource.GenericObjectNode;
import de.willuhn.datasource.pseudo.PseudoIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Item;
import de.willuhn.jameica.gui.NavigationItem;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * QRCodeScanner Navigation Items
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class QrCodeScannerItem implements NavigationItem {

	private NavigationItem parent = null;

	private Action action;

	private String navitext;

	private ArrayList<Item> children;

	private String icon;

	private Boolean enabled;

	/**
	 * Erstellt ein QrCodeScanner Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die KLasse, die das das Interface <b>Action</b> implemtiert
	 *            und durch das NavigationItem ausgefhrt werden soll.
	 */
	public QrCodeScannerItem(NavigationItem item, String navitext, Action action) {
		this(item, navitext, action, null);
	}

	/**
	 * Erstellt ein QrCodeScanner Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die KLasse, die das das Interface <b>Action</b> implemtiert
	 *            und durch das NavigationItem ausgefhrt werden soll.
	 * @param icon
	 *            String des Icon-Bildes des NavigationItems
	 */
	public QrCodeScannerItem(NavigationItem item, String navitext,
			Action action, String icon) {
		this(item, navitext, action, icon, true);
	}

	/**
	 * Erstellt ein QrCodeScanner Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die KLasse, die das das Interface <b>Action</b> implemtiert
	 *            und durch das NavigationItem ausgefhrt werden soll.
	 * @param icon
	 *            String des Icon-Bildes des NavigationItems
	 * @param enabled setzt den Aktiv-Status des Items
	 */
	public QrCodeScannerItem(NavigationItem item, String navitext,
			Action action, String icon, Boolean enabled) {
		this.parent = item;
		this.action = action;
		this.navitext = navitext;
		this.icon = icon;
		this.enabled = enabled;
		children = new ArrayList<Item>();
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.Item#getName()
	 */
	@Override
	public String getName() {
		return navitext;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.Item#getAction()
	 */
	@Override
	public Action getAction() throws RemoteException {
		return action;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.Item#addChild(de.willuhn.jameica.gui.Item)
	 */
	@Override
	public void addChild(Item i) throws RemoteException {
		throw new RemoteException("Keine Kinder erlaubt");
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.Item#isEnabled()
	 */
	@Override
	public boolean isEnabled() throws RemoteException {
		return enabled;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.Item#setEnabled(boolean, boolean)
	 */
	@Override
	public void setEnabled(boolean enabled, boolean recursive)
			throws RemoteException {
		this.enabled = enabled;

	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObjectNode#getChildren()
	 */
	@Override
	public GenericIterator getChildren() throws RemoteException {
		return PseudoIterator.fromArray(children
				.toArray(new QrCodeScannerItem[children.size()]));
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObjectNode#getParent()
	 */
	@Override
	public GenericObjectNode getParent() throws RemoteException {
		return parent;
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObjectNode#getPath()
	 */
	@Override
	public GenericIterator getPath() throws RemoteException {
		@SuppressWarnings("unchecked")
		List<NavigationItem> list = PseudoIterator.asList(this.parent.getPath());
		list.add(this);
		return PseudoIterator.fromArray((NavigationItem[]) list
				.toArray(new NavigationItem[list.size()]));
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObjectNode#getPossibleParents()
	 */
	@Override
	public GenericIterator getPossibleParents() throws RemoteException {
		throw new RemoteException("not implemented");
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see
	 * de.willuhn.datasource.GenericObjectNode#hasChild(de.willuhn.datasource
	 * .GenericObjectNode)
	 */
	@Override
	public boolean hasChild(GenericObjectNode arg0) throws RemoteException {
		return false;
	}

	/**
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObject#equals(de.willuhn.datasource.
	 * GenericObject)
	 */
	@Override
	public boolean equals(final GenericObject arg0) throws RemoteException {
		if (arg0 == null || !(arg0 instanceof QrCodeScannerItem))
			return false;
		return this.getID().equals(arg0.getID());
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObject#getAttribute(java.lang.String)
	 */
	@Override
	public Object getAttribute(String arg0) throws RemoteException {
		return getName();
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObject#getAttributeNames()
	 */
	@Override
	public String[] getAttributeNames() throws RemoteException {
		return new String[] { "name" };
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObject#getID()
	 */
	@Override
	public String getID() {
		return getClass().getName() + "." + getName();
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "name";
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.extension.Extendable#getExtendableID()
	 */
	@Override
	public String getExtendableID() {
		return getID();
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.NavigationItem#getIconOpen()
	 */
	@Override
	public Image getIconOpen() throws RemoteException {
		return SWTUtil.getImage(icon != null ? icon : "webcam_add.png");
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.NavigationItem#getIconClose()
	 */
	@Override
	public Image getIconClose() throws RemoteException {
		return SWTUtil.getImage(icon != null ? icon : "webcam_add.png");
	}

	/*
	 * (nicht-Javadoc)
	 * 
	 * @see de.willuhn.jameica.gui.NavigationItem#isExpanded()
	 */
	@Override
	public boolean isExpanded() throws RemoteException {
		return false;
	}

}
