/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * Erstellt ein ImageLable basierend auf einem SWT Label, das zur Darstellung
 * von Text und Bilder benutzt werden kann. Die Gre wird entweder durch ein
 * Bild oder ein Grengabe im Konstruktor gesetzt und ist dann nicht mehr
 * vernderbar.
 * 
 * @author munstfr
 * @version 1.1
 * 
 */
public class ImageLabelPart implements Part, PaintListener {

	/**
	 * Image das angezeigt werden soll
	 */
	private Image img = null;
	/**
	 * Text, der angezeigt werden soll
	 */
	private String text = null;
	/**
	 * Gre des Lables
	 */
	private Point size = null;
	/**
	 * Das Label zum anziegen des Images/Textes
	 */
	private Label lbl = null;

	/**
	 * Erstellt das ImageLable ohne weitere Inhalte
	 */
	public ImageLabelPart() {
	}

	/**
	 * Erstellt eine ImageLabel mit Image und setzt die Gre des Labels auf die
	 * Bildgre
	 * 
	 * @param img
	 *            Image das dargestellt werden soll.
	 */
	public ImageLabelPart(Image img) {
		this.img = img;
	}

	/**
	 * Erstellt eine ImageLabel mit leeren Inhalt in der angegebenen Gre.
	 * 
	 * @param size
	 *            Point-Object mit Grenangaben
	 * 
	 */
	public ImageLabelPart(Point size) {
		this.size = size;
	}

	/**
	 * Erstellt eine ImageLabel mit Bild in der angegebenen Gre.
	 * 
	 * @param img
	 *            Image das dargestellt werden soll.
	 * @param size
	 *            Point-Object mit Grenangaben
	 */
	public ImageLabelPart(Image img, Point size) {
		this.img = img;
		this.size = size;
	}

	/**
	 * Setzt ein neues Image und zeigt es an
	 * 
	 * @param img IMage, dass angezeigt werden soll
	 */
	public void setImage(Image img) {
		this.img = img;

		// draw image if lbl exists
		if (lbl != null) {
			final Image a = this.img;
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					// ... do any work that updates the screen ...
					if (!lbl.isDisposed()) {
						lbl.setImage(a);
						lbl.redraw();
						lbl.update();
						lbl.getParent().layout();
					}
				}
			});
		}
	}

	/**
	 * Setzt einen neuen Text, der angezeigt werden soll.
	 * Der Text ersetzt auch ein evtl. angezeigtes Image.
	 * 
	 * @param text der angezeigt werden soll
	 */
	public void setText(String text) {
		this.text = text;

		// draw image if lbl exists
		if (lbl != null) {
			final String t = this.text;
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					// ... do any work that updates the screen ...
					if (!lbl.isDisposed()) {
						lbl.setText(t);
						lbl.redraw();
						lbl.update();
						lbl.getParent().layout();
					}
				}
			});
		}
	}

	/**
	 * Gibt die Gre des ImageLabels als Point zurck
	 * 
	 * @return Gre des ImageLables
	 */
	public Point getSize() {
		return size;
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	public void paint(Composite parent) throws RemoteException {
		// final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
		// | GridData.FILL_VERTICAL);
		if (parent == null) {
			Logger.error("Parent is null");
		}

		Composite container = new Composite(parent, SWT.BORDER);
		lbl = new Label(container, SWT.WRAP);

		if (getSize() != null) {
			lbl.setSize(getSize());
		}
		if (img != null) {
			lbl.setImage(img);
		}

		GridData labelVerticalyAlignedGridData = new GridData();
		labelVerticalyAlignedGridData.verticalAlignment = SWT.CENTER;
		labelVerticalyAlignedGridData.grabExcessVerticalSpace = true;
		labelVerticalyAlignedGridData.horizontalAlignment = SWT.CENTER;
		labelVerticalyAlignedGridData.grabExcessHorizontalSpace = true;
		lbl.setLayoutData(labelVerticalyAlignedGridData);
	
	}

	@Override
	public void paintControl(PaintEvent e) {
		Logger.debug(this.getClass().getName() + " - paintControl");
		if (img != null)
			e.gc.drawImage(img, 0, 0);
	}

}

/***********************************************************
 * History:
 * $Log ImageLabelPart.java,v$
 * $Date:: 2016-07-08 13:59:30#$ Date of last commit
 * 
 * Revision 1.1 Date: 2014-04-01
 *   Anpassungen fr Drop&ffnen von PDF Dateien hinzugefgt
 *      
 *********************************************************/
