/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.parts;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamDevice;

import de.frankmuenster.jameica.qrscanner.WebcamWatchdog;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.WebcamSelectedListener;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.logging.Logger;

public class WebcamSelector implements Part {
	
	/**
	 * Unsere Listeners fr Webcam-nderungen
	 */
	private final Set<WebcamSelectedListener> listeners = new CopyOnWriteArraySet<WebcamSelectedListener>();
	
	
	/**
	 * Fuegt eine Listener hinzu
	 * 
	 * @param listener
	 *            WebcamChangeListener der hinzugefgt werden soll
	 */
	public final void addListener(final WebcamSelectedListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernt einen Listener aus der Liste
	 * 
	 * @param listener
	 *            WebcamChangeListener der entfernt werden soll
	 */
	public final void removeListener(final WebcamSelectedListener listener) {
		listeners.remove(listener);
	}

	/**
	 * 
	 * Benachrichtigen der registrierten Listeners.
	 * 
	 */
	private final void notifyListeners() {
		for (WebcamSelectedListener listener : listeners) {
			listener.notifyOfWebcamsChanged();;
		}
	}
	

	/**
	 * Das Dropdown-Select-Element mit alle Webcams
	 */
	private SelectInput webcamList;
	
	private WebcamDevice defaultWebcam;
	
	public WebcamSelector () {
		
		defaultWebcam = Webcam.getDefault().getDevice();
	}
	
	public String getValue() {
		return (String) webcamList.getValue();
	}
	
	
	@Override
	public void paint(Composite parent) throws RemoteException {
		if (parent == null) {
			Logger.error("Parent is null");
		}

		Composite container = new Composite(parent, SWT.NONE);
		
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		container.setLayout(gridLayout);
		
		if (webcamList == null ) {
			webcamList = new SelectInput(getWebcamNames(), defaultWebcam.getName());
		}
		
		webcamList.paint(container);
		webcamList.getControl().addListener(SWT.Selection, new Listener() {
			
			@Override
			public void handleEvent(Event arg0) {
				notifyListeners();
			}
		});		
		
	}

	private List<String> getWebcamNames() {
		
		//Aktualisiere Webcamliste
		Webcam.resetDriver();
		Webcam.setDriver(WebcamWatchdog.getDriver());
		Webcam.getDefault();
		
		List<String> webcams = new ArrayList<>();
		int i = 1;
		for (WebcamDevice webcam : Webcam.getDriver().getDevices()) {
			Logger.debug("Webcam ("+ (i++) +"): "  + webcam.getName() );
			webcams.add(webcam.getName());
		}
		
		return webcams;
		
	}
	

	
}
