/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.qrscanner.gui.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.controller.SettingsController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * View fr die Einstellungen
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SettingsView extends AbstractView {

	private final static I18N I18N = Settings.i18n();

	private SettingsController control;
	private TabFolder folder = null;

	/**
	 * Liefert eine neu Settings-View zurck und erstellt einen passenden
	 * Controller
	 */
	public SettingsView() {

		this.control = new SettingsController(this);
	}

	@Override
	public void bind() throws Exception {

		GUI.getView().setTitle(I18N.tr("Settings"));

		GridLayout grid = new GridLayout(2, false);
		grid.verticalSpacing = 10;

		TabGroup grundEinstellung = new TabGroup(getTabFolder(),
				I18N.tr("Basic Settings"));

		ColumnLayout cl = new ColumnLayout(grundEinstellung.getComposite(), 2);
		SimpleContainer left = new SimpleContainer(cl.getComposite());
		SimpleContainer right = new SimpleContainer(cl.getComposite());

		left.addHeadline(I18N.tr("IP Streaming Support"));
		left.addLabelPair(I18N.tr("Use DroidCam App"), control.getUseDroidCam());

		ColumnLayout cl1 = new ColumnLayout(left.getComposite(), 2);

		SimpleContainer host = new SimpleContainer(cl1.getComposite());
		host.addLabelPair("DroidCam Host and Port", control.getDroidCamHost());
		SimpleContainer port = new SimpleContainer(cl1.getComposite());
		port.addInput(control.getDroidCamPort());

		left.addText("     ", false);
		left.addHeadline(I18N.tr("Webcams found"));
		left.addPart(control.getDiscoverdWebcams());
		left.addText("     ", false);
		left.addPart(control.getRefeshBtn());

		right.addText("       ", false);

		ButtonArea buttons = new ButtonArea();

		buttons.addButton(I18N.tr("Save"), new Action() {
			public void handleAction(Object context)
					throws ApplicationException {
				control.speichern();
			}
		}, null, true, "document-save.png");
		buttons.paint(getParent());

	}

	/**
	 * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen
	 * untergebracht sind.
	 * 
	 * @return der Tab-Folder.
	 */
	private TabFolder getTabFolder() {
		if (this.folder != null)
			return this.folder;

		this.folder = new TabFolder(getParent(), SWT.NONE);
		this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		return this.folder;
	}

}
