//
// Copyright 2014 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.jameica.qrscanner.jmdns;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;

import javax.jmdns.JmDNS;
import javax.jmdns.ServiceEvent;
import javax.jmdns.ServiceInfo;
import javax.jmdns.ServiceListener;

import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.willuhn.logging.Logger;

/**
 * Listener zum berwachen von neuem Webcams, die sich per
 * mDNS anmelden (Iphone Mini-Webcam).
 *  
 * @author munstfr
 * @version 1.0
 * 
 */
public final class IpWebcamListener implements ServiceListener {
	
	
	/**
	 * Das JmDNS Objekt.
	 */
	private static JmDNS jmdns;
	
	private IpWebcamListener() {
		// Leerer Konstruktor
	}
	
	/**
	 * Erstellt genau eine Instanz dieser Klasse und des jmdns Objekts und fgt
	 * einen Listener fr die IPhone Mini-Webcam-App hinzu. 
	 */
	public static synchronized void startIpWebcamListener() {
		
		if (jmdns != null) {
			return;
		}

		try {
			jmdns = JmDNS.create(InetAddress.getLocalHost());
			//MiniWabcam iphone
			jmdns.addServiceListener("_mini_webcam_101._tcp.local.", new IpWebcamListener());
			
		} catch (UnknownHostException e) {
			Logger.error("Der lokale Host kann nicht ermittelt werden. Der JmDNS Listener "
					+ "wird nicht funktionieren." + e.getMessage());
		} catch (IOException e) {
			Logger.error("Die JmDNS Instanze konnte nicht erstellt werden. Der JmDNS Listener "
					+ "wird nicht funktionieren." + e.getMessage());
		}
	
	}
	
	/**
	 * Liefert das JmDNS-Objekt.
	 * 
	 * @return das JmDNS-Objekt. 
	 */
	public static JmDNS getJmDNS() {
		return jmdns;
	}

	private int test1=0;
	
	/**
	 * @see javax.jmdns.ServiceListener#serviceAdded(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceAdded(ServiceEvent event) {
		Logger.debug("Service added: " + event.getInfo());
		ServiceInfo serviceInfo = event.getInfo();
		
		if (serviceInfo.getProtocol().equals("_mini_webcam_101")) {
			try {
				IpCamDeviceRegistry.register(new IpCamDevice("Lignano Beach", "http://88.37.116.138/mjpg/video.mjpg", IpCamMode.PUSH));
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		
		if (test1 == 0) {
			try {
				IpCamDeviceRegistry.register(new IpCamDevice("Lignano Beach", "http://88.37.116.138/mjpg/video.mjpg", IpCamMode.PUSH));
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			test1 = 1;
		}
		
		
	}

	/**
	 * @see javax.jmdns.ServiceListener#serviceRemoved(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceRemoved(ServiceEvent event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("Service removed: " + event.getInfo());
		

	}

	/**
	 * @see javax.jmdns.ServiceListener#serviceResolved(javax.jmdns.ServiceEvent)
	 */
	@Override
	public void serviceResolved(ServiceEvent event) {
		// TODO Automatisch generierter Methodenstub
		Logger.debug("Service resolved: " + event.getInfo());
	}

}
