/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.qrscanner.utils;

import java.util.HashSet;
import java.util.Set;

/**
 * ContryCode Klasse zum validiern, ob IBAN udn BIC zu EUCountry gehoeren. Aus
 * Forumeintrag http://www.tutorials.de/java/259520-iban-bic.html
 * 
 * @see <a
 *      href="www.tutorials.de/java/259520-iban-bic.html">http://www.tutorials.de/java/259520-iban-bic.html</a>
 * 
 * @author takidoso / munstfr
 * @version 1.0
 * 
 */
public final class EUCountryCodes {

	/**
	 * Liste der Lnder fr einen gltiger IBAN Code
	 */
	static private Set<String> ibanCountryCodes = new HashSet<String>();
	/**
	 * Liste der Lnder fr einen gltiger BIC Code
	 */
	static private Set<String> bicCountryCodes = new HashSet<String>();

	static {
		ibanCountryCodes.add("AT");
		ibanCountryCodes.add("BE");
		ibanCountryCodes.add("CY");
		ibanCountryCodes.add("CZ");
		ibanCountryCodes.add("DE");
		ibanCountryCodes.add("DK");
		ibanCountryCodes.add("EE");
		ibanCountryCodes.add("ES");
		ibanCountryCodes.add("FI");
		ibanCountryCodes.add("FR");
		ibanCountryCodes.add("GB");
		ibanCountryCodes.add("GI");
		ibanCountryCodes.add("GR");
		ibanCountryCodes.add("HU");
		ibanCountryCodes.add("IE");
		ibanCountryCodes.add("IS");
		ibanCountryCodes.add("IT");
		ibanCountryCodes.add("LI");
		ibanCountryCodes.add("LT");
		ibanCountryCodes.add("LU");
		ibanCountryCodes.add("LV");
		ibanCountryCodes.add("MT");
		ibanCountryCodes.add("NL");
		ibanCountryCodes.add("NO");
		ibanCountryCodes.add("PL");
		ibanCountryCodes.add("PT");
		ibanCountryCodes.add("SE");
		ibanCountryCodes.add("SI");
		ibanCountryCodes.add("SK");

		bicCountryCodes.add("AT");
		bicCountryCodes.add("BE");
		bicCountryCodes.add("CY");
		bicCountryCodes.add("CZ");
		bicCountryCodes.add("DE");
		bicCountryCodes.add("DK");
		bicCountryCodes.add("EE");
		bicCountryCodes.add("ES");
		bicCountryCodes.add("FI");
		bicCountryCodes.add("FR");
		bicCountryCodes.add("GB");
		bicCountryCodes.add("IE");
		bicCountryCodes.add("GF");
		bicCountryCodes.add("GI");
		bicCountryCodes.add("GP");
		bicCountryCodes.add("GR");
		bicCountryCodes.add("HU");
		bicCountryCodes.add("IE");
		bicCountryCodes.add("IS");
		bicCountryCodes.add("IT");
		bicCountryCodes.add("LI");
		bicCountryCodes.add("LT");
		bicCountryCodes.add("LU");
		bicCountryCodes.add("LV");
		bicCountryCodes.add("MQ");
		bicCountryCodes.add("MT");
		bicCountryCodes.add("NL");
		bicCountryCodes.add("NO");
		bicCountryCodes.add("PL");
		bicCountryCodes.add("PT");
		bicCountryCodes.add("RE");
		bicCountryCodes.add("SE");
		bicCountryCodes.add("SI");
		bicCountryCodes.add("SK");
	}

	private EUCountryCodes() {
	}

	/**
	 * Ueberprueft Eingabestring (im BIC-Format) auf Laenderkennung an Stelle 5
	 * und 6.
	 * 
	 * @param bic
	 *            <Sting> zu ueberpruefende BIC
	 * @return <b>true</b>, wenn EU Laenderkennung gefunden, <b>false</b> sonst.
	 */
	static public boolean isBicEUCountry(final String bic) {
		return bicCountryCodes.contains(bic.substring(4, 6));
	}

	/**
	 * Ueberprueft Eingabestring (im IBAN-Format) auf Laenderkennung an Stelle 1
	 * und 2.
	 * 
	 * @param iban
	 *            <Sting> zu berprfende IBAN
	 * @return <b>true</b>, wenn EU Lnderkennung gefunden, <b>false</b> sonst.
	 */
	static public boolean isIbanEUCountry(final String iban) {
		return ibanCountryCodes.contains(iban.substring(0, 2));
	}
}
